/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// I'm trying to comment the source but my english isn't perfect, so be tolerant 
// and please correct all the mistakes you find. ;)

#include <GL/glut.h>

#include "stellarium.h"
#include "star_mgr.h"
#include "draw.h"
#include "constellation.h"
#include "constellation_mgr.h"
#include "planet_mgr.h"
#include "nebula.h"
#include "nebula_mgr.h"
#include "texture.h"
#include "stellarium_ui.h"
#include "parsecfg.h"
#include "navigation.h"
#include "s_gui.h"

using namespace std;

stellariumParams global;
Star_mgr VouteCeleste;				// Class to manage all the stars from HR catalog
Constellation_mgr ConstellCeleste;	// Class to manage the constellation boundary and name
Nebula_mgr messiers;				// Class to manage the messier objects
Planet_mgr SolarSystem;				// Class to manage the planets
int texIds[200];					// Textures numbers

void init();

// ***************  Print a beautiful console logo !! ******************
void drawIntro(void)
{	printf("\n");
	printf("    -----------------------------------------\n");
	printf("    | ## ### ### #   #   ### ###  #  # # # #|\n");
	printf("    | #   #  ##  #   #   ### ##   #  # # ###|\n");
	printf("    |##   #  ### ### ### # # # #  #  ### # #|\n");
	printf("    |                     %s|\n",APP_NAME);
	printf("    -----------------------------------------\n\n");
	printf("Copyright (C) 2002 Fabien Chereau chereau@free.fr\n\n");
	printf("Please send bug report & comments to stellarium@free.fr\n\n");
};

// ************************  Main display loop  ************************
void glutDisplay(void)
{	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	gluPerspective(global.Fov, (double) global.X_Resolution / global.Y_Resolution, 1, 1000);
	glMatrixMode(GL_MODELVIEW);

	// Update the position of observation and time etc...
	Update_variables();
	Update_time(SolarSystem);
	ComputeZenith();

	// Execute all the drawing function in the correct order from the more 
	// far to nearest objects
	
	// ---- Equatorial Coordinates
	Switch_to_equatorial();							// Switch in Equatorial coordinates
	DrawMilkyWay();									// Draw the milky way --> init the buffers
	messiers.Draw();								// Draw the Messiers Objects
	if (global.FlagConstellationDrawing && !global.FlagRealMode)
		ConstellCeleste.Draw();						// Draw all the constellations
	if (global.FlagStars) VouteCeleste.Draw();		// Draw the stars
	if (global.FlagPlanets) SolarSystem.Draw();		// Draw the planets
	if (global.FlagEquatorialGrid && !global.FlagRealMode)
	{	DrawMeridiens();							// Draw the meridian lines
		DrawParallels();							// Draw the parallel lines
	}
	if (global.FlagEquator && !global.FlagRealMode) DrawEquator();			// Draw the celestial equator line
	if (global.FlagEcliptic && !global.FlagRealMode) DrawEcliptic();		// Draw the ecliptic line
	if (global.FlagConstellationName && !global.FlagRealMode)				// Draw the names of the constellations
		ConstellCeleste.DrawName();	
	if (global.FlagSelect && !global.FlagRealMode)							// Draw the star pointer
		DrawPointer(global.Pointer,global.SelectedObject.Size*10/global.Fov,global.SelectedObject.RGB,global.SelectedObject.type);

	// ---- AltAzimutal Coordinates
	Switch_to_altazimutal();						// Switch in AltAzimutal coordinates
	if (global.FlagAzimutalGrid && !global.FlagRealMode)
	{	DrawMeridiensAzimut();						// Draw the "Altazimuthal meridian" lines
		DrawParallelsAzimut();						// Draw the "Altazimuthal parallel" lines
	}
	if (global.FlagAtmosphere) DrawAtmosphere();	// Draw the blue sky
	if (global.FlagHorizon && global.FlagGround)	
		DrawDecor(2);								// Draw the mountains
	if (global.FlagGround) DrawGround();			// Draw the ground
	if (global.FlagFog) DrawFog();					// Draw the fog
	if (global.FlagCardinalPoints && !global.FlagRealMode) DrawCardinaux();	// Daw the cardinal points

	// ---- 2D Displays
	renderUi();

	glutSwapBuffers();								// Swap the 2 buffers
}

// ************************  On resize  *******************************
void glutResize(int w, int h)
{	if (!h) return;
	glViewport(0, 0, global.X_Resolution, global.Y_Resolution);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(global.Fov, (double) global.X_Resolution / global.Y_Resolution, 1, 10000);	// Update the ratio
	glutPostRedisplay();
	glMatrixMode(GL_MODELVIEW);
}

// *******************  Handle the Flag keys  **************************
void processNormalKeys(unsigned char key, int x, int y) 
{	switch (key) 			// toggle the different functions
	{	case 27 : exit(0); break;
	}
	HandleNormalKey(key,GUI_DOWN);
}

// ************************  Initialisation  ******************************
void loadCommonTextures(void)
{	printf("Loading textures...\n");
	texIds[1] = createTexture("sol2");
	texIds[2] = createTexture("voielactee256x256");
	texIds[3] = createTexture("horizon3");
	texIds[4] = createTexture("ciel");	
	texIds[6] = createTexture("n");
	texIds[7] = createTexture("s");
	texIds[8] = createTexture("e");
	texIds[9] = createTexture("w");
	texIds[10]= createTexture("zenith");
	texIds[11]= createTexture("nadir");
	texIds[12]= createTexture("pointeur2");
	texIds[25]= createTexture("etoile32x32");
	texIds[26]= createTexture("pointeur4");
	texIds[27]= createTexture("pointeur5");
	texIds[30]= createTexture("spacefont");
	texIds[31]= createTexture("montagne1");
	texIds[32]= createTexture("montagne2");
	texIds[33]= createTexture("montagne3");
	texIds[34]= createTexture("montagne4");
	texIds[40]= createTexture("sun");
	texIds[41]= createTexture("mercure");
	texIds[42]= createTexture("venus");
	texIds[44]= createTexture("mars");
	texIds[45]= createTexture("jupiter");
	texIds[46]= createTexture("saturn");
	texIds[47]= createTexture("saturneAnneaux128x128");
	texIds[48]= createTexture("halo");
	texIds[49]= createTexture("lune");
	texIds[50]= createTexture("haloLune");
	if (messiers.ReadTexture()==0) printf("Error while loading messier Texture\n");
}

// ********************  Handle the special keys  **********************
void pressKey(int key, int, int) 
{	switch (key) 
	{	// Direction and zoom deplacements
		case GLUT_KEY_LEFT :		global.deltaRa = -1;	break;
		case GLUT_KEY_RIGHT :		global.deltaRa =  1;	break;
		case GLUT_KEY_UP :			global.deltaDe =  1;	break;
		case GLUT_KEY_DOWN :		global.deltaDe = -1;	break;
		case GLUT_KEY_PAGE_UP :		global.deltaFov= -1;	break;
		case GLUT_KEY_PAGE_DOWN :	global.deltaFov=  1;	break;
	}
}

// *******************  Stop mooving and zooming  **********************
void releaseKey(int key, int, int)
{	switch (key)
	{	// When a deplacement key is released stop mooving
		case GLUT_KEY_LEFT :
		case GLUT_KEY_RIGHT :		global.deltaRa = 0;
									break;
		case GLUT_KEY_UP : 
		case GLUT_KEY_DOWN :		global.deltaDe = 0;
									break;
		case GLUT_KEY_PAGE_UP :
		case GLUT_KEY_PAGE_DOWN :	global.deltaFov = 0;
									break;
	}
}

// ***************  Handle the mouse deplacement  *********************
void MoveMouse(int x, int y)
{
	HandleMove(x,y);
}

// *******************  Handle the mouse clic  *************************
void Mouse(int button, int state, int x, int y)
{
	HandleClic(x,y,state,button);
}

// *******************  Read the conuration file  *************************
void loadParams(void)
{	global.SkyBrightness=0;
	global.Fov=60.;
	global.X_Resolution=800;
	global.Y_Resolution=600;
	global.RaZenith=0;
	global.DeZenith=0;
	global.AzVision=0;
	global.AltVision=0;
	global.deltaFov=0;
	global.deltaDe=0;
	global.deltaRa=0;
	global.Fps=40;
	global.FlagSelect=false;
	global.FlagRealMode=false;

	double tempLatitude=0;
	double tempLongitude=0;
	char * tempDate = NULL;
	char * tempTime = NULL;

	printf("Loading configuration file...\n");
	// array of cfgStruct for reading on config file
	cfgStruct cfgini[] = 
	{// parameter				type		address of variable
		{"X_RESOLUTION",		CFG_INT,	&global.X_Resolution},
		{"Y_RESOLUTION",		CFG_INT,	&global.Y_Resolution},
		{"FULLSCREEN",			CFG_BOOL,	&global.Fullscreen},
		{"BBP_MODE",			CFG_INT,	&global.bppMode},
		{"FLAG_FPS",			CFG_BOOL,	&global.FlagFps},		
		{"FLAG_STARS",			CFG_BOOL,	&global.FlagStars},
		{"FLAG_PLANETS",		CFG_BOOL,	&global.FlagPlanets},
		{"FLAG_NEBULA",			CFG_BOOL,	&global.FlagNebula},
		{"FLAG_GROUND",			CFG_BOOL,	&global.FlagGround},
		{"FLAG_HORIZON",		CFG_BOOL,	&global.FlagHorizon},
		{"FLAG_FOG",			CFG_BOOL,	&global.FlagFog},
		{"FLAG_ATMOSPHERE",		CFG_BOOL,	&global.FlagAtmosphere},
		{"FLAG_CONSTELLATION_DRAWING",	CFG_BOOL,   &global.FlagConstellationDrawing},
		{"FLAG_CONSTELLATION_NAME",		CFG_BOOL,   &global.FlagConstellationName},
		{"FLAG_AZIMUTAL_GRID",	CFG_BOOL,   &global.FlagAzimutalGrid},
		{"FLAG_EQUATORIAL_GRID",CFG_BOOL,   &global.FlagEquatorialGrid},
		{"FLAG_EQUATOR",		CFG_BOOL,   &global.FlagEquator},
		{"FLAG_ECLIPTIC",		CFG_BOOL,   &global.FlagEcliptic},
		{"FLAG_CARDINAL_POINTS",CFG_BOOL,   &global.FlagCardinalPoints},
		{"FLAG_REAL_MODE",		CFG_BOOL,   &global.FlagRealMode},
		{"FLAG_REAL_TIME",		CFG_BOOL,   &global.FlagRealTime},
		{"FLAG_ACCELERED_TIME",	CFG_BOOL,   &global.FlagAcceleredTime},
		{"FLAG_VERY_FAST_TIME",	CFG_BOOL,   &global.FlagVeryFastTime},
		{"FLAG_MENU",			CFG_BOOL,   &global.FlagMenu},
		{"FLAG_HELP",			CFG_BOOL,   &global.FlagHelp},
		{"FLAG_INFOS",			CFG_BOOL,   &global.FlagInfos},
		{"LATITUDE",			CFG_DOUBLE,	&tempLatitude},
		{"LONGITUDE",			CFG_DOUBLE,	&tempLongitude},
		{"DATE",				CFG_STRING,	&tempDate},
		{"TIME",				CFG_STRING,	&tempTime},
		{NULL, CFG_END, NULL}	/* no more parameters */
	};

	if (cfgParse("./config.txt", cfgini, CFG_SIMPLE) == -1)
	{	printf("An error was detected in the config file\n");
		return;
	}

	// init the time parameters with current time and date
	int d,m,y,hour,min,sec;
	time_t rawtime;
	tm * ptm;
	time ( &rawtime );
	ptm = gmtime ( &rawtime );
	y=ptm->tm_year+1900;
	m=ptm->tm_mon+1;
	d=ptm->tm_mday;
	hour=ptm->tm_hour;
	min=ptm->tm_min;
	sec=ptm->tm_sec;

	// If no date given -> default today
	if (tempDate!=NULL && (sscanf(tempDate,"%d/%d/%d\n",&m,&d,&y) != 3))
	{	printf("ERROR, bad date format : please change config.txt\n\n");
		exit(1);
	}
	if (tempTime!=NULL && (sscanf(tempTime,"%d:%d:%d\n",&hour,&min,&sec) != 3))
	{	printf("ERROR, bad time format : please change config.txt\n\n");
		exit(1);
	}

	if (m>12 || m<1 || d<1 || d>31)
	{	printf("ERROR, bad month value : please change config.txt\n\n");
		exit(0);
	}

	if (hour>23 || hour<0 || min<0 || min>59 || sec<0 || sec>59)
	{	printf("ERROR, bad time value : please change config.txt\n\n");
		exit(0);
	}

	if (tempDate!=NULL) delete tempDate;
	if (tempTime!=NULL) delete tempTime;	

	// calc the julian date
	global.JDay=DateOps::dmyToDay(d,m,y);	// store it in the global variable JDay
	global.JDay+=((double)hour*0.0416666667+((double)min+(double)sec/60)*0.0006944444);


	// arrange the latitude/longitude to fit with astroOps conventions
	tempLatitude=90.-tempLatitude;
	tempLongitude=-tempLongitude;
	if (tempLongitude<0) tempLongitude=360.+tempLongitude;
	// Set the change in the position
	global.ThePlace.setLongitude(tempLongitude);
	global.ThePlace.setLatitude(tempLatitude);

}

// ****************  Initialisation of glut and openGL  ****************
void init() 
{	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(pressKey);
	glutSpecialUpFunc(releaseKey);
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutDisplay);
	glutReshapeFunc(glutResize);
	glutMouseFunc(Mouse);
	glutMotionFunc(MoveMouse);
	glutPassiveMotionFunc(MoveMouse);
}

// ***************************  Main  **********************************
void main (void)
{	drawIntro();											// Print the console logo
	loadParams();											// Load the params from config.txt

	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);

	if (global.Fullscreen)									// FullScreen Mode
	{	char str[20];										// Init screen size
		sprintf(str,"%dx%d:%d",global.X_Resolution,global.Y_Resolution,global.bppMode);
		glutGameModeString(str);							// define resolution, color depth
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))		// enter full screen
		{	glutEnterGameMode();
		}
		else												// Error
		{	printf("\n\nERROR : Unsuported screen mode, change the config.txt file\n");
			exit(1);
		}
	}
	else													// Windowed mode
	{	glutCreateWindow(APP_NAME);
		glutFullScreen();
		global.X_Resolution = glutGet(GLUT_SCREEN_WIDTH);
		global.Y_Resolution = glutGet(GLUT_SCREEN_HEIGHT);  // Windowed fullscreen mode
//		glutReshapeWindow(global.X_Resolution, global.Y_Resolution);
	}

	glutIgnoreKeyRepeat(1);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);			// init the blending function parameters (read the doc to understand it ;) ...)

	init();													// Set the callbacks

	initTextures();											// Init the texture manager
	loadCommonTextures();									// Load the common used textures
	VouteCeleste.Load(DATA_DIR"/catalog.fab");					// Load stars
															// Load constellations 
	ConstellCeleste.Load(DATA_DIR"/constellationsmaj.fab",VouteCeleste);		
	SolarSystem.Compute(global.JDay,global.ThePlace);		// Compute planet data
	InitMeriParal();										// Precalculation for the grids drawing
	initUi();												// initialisation of the User Interface
	messiers.Read(DATA_DIR"/messier.fab");						// read the messiers object data

	glutMainLoop();											// Start drawing
}
