/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "navigation.h"
#include "stellarium.h"

/*******************  My coordinate conventions :  *******************

Always count the RA angle from z axis to x axis 

		y
		|
		|   +A
		|DE.|
		| . |
		|___|____x
       / .  | .
     / RA . |.
   /...... .|
 z
						*/

// *******  Calc the Zenith position for the location and Time  ********
void ComputeZenith(void)
{	global.RaZenith = -AstroOps::greenwichSiderealTime(global.JDay) + global.ThePlace.longitude();
	global.RaZenith =  AstroOps::normalizeRadians(global.RaZenith);
	global.DeZenith =  Astro::PI_OVER_TWO - global.ThePlace.latitude();
}

// **** Increment/decrement smoothly the vision field and position *****
void Update_variables(void)
{	// to prevent strange behaviour, change the speed of mooving EVERY FRAMES
	// depending of the field of view : more it is zoomed, more the mooving 
	// speed is low (in angle)
	float depl=(float)0.6/(global.Fps+1);
	if (global.deltaRa<0)
	{	global.deltaRa = -depl*global.Fov/30;
	}
	else 
	{	if (global.deltaRa>0)
		{	global.deltaRa = depl*global.Fov/30;
		}
	}
	if (global.deltaDe<0) 
	{	global.deltaDe = -depl*global.Fov/30;
	}
	else 
	{	if (global.deltaDe>0)
		{	global.deltaDe = depl*global.Fov/30;
		}
	}
	if (global.deltaFov<0)
	{	global.deltaFov = -depl*global.Fov*5;
	}
	else 
	{	if (global.deltaFov>0)
		{	global.deltaFov = depl*global.Fov*5;
		}
	}

	// if we are zooming in or out
	if (global.deltaFov)
	{	if (global.Fov+global.deltaFov>0.0001 && global.Fov+global.deltaFov<100)					// change current zoom
			global.Fov+=global.deltaFov;
	}
	// if we are mooving in the Azimthal angle (left/right)
	if (global.deltaRa) global.AzVision-=global.deltaRa;
	if (global.deltaDe)
	{	if (global.AltVision+global.deltaDe <= PI/2 && global.AltVision+global.deltaDe >= -PI/2)
			global.AltVision+=global.deltaDe;
	}
}

// *********************************************************************
// Increment time every second for real time mode and every frame for accelered time mode
void Update_time(Planet_mgr &_SolarSystem)
{	global.Frame++;
	int DeltaTemps=glutGet(GLUT_ELAPSED_TIME)-global.Timefr;
	global.Timefr+=DeltaTemps;
	if (global.Timefr-global.Timebase > 1000) 
	{	global.Fps=global.Frame*1000.0/(global.Timefr-global.Timebase);		// Calc the PFS rate
		global.Frame = 0;
		global.Timebase+=1000;
		_SolarSystem.Compute(global.JDay,global.ThePlace);
	}
	if (global.FlagRealTime)								// real time animation
	{	global.JDay+=SECONDE*(float)DeltaTemps/1000.0;
	}
	if (global.FlagAcceleredTime)							// accelered time animation
	{	global.JDay+=MINUTE;
		_SolarSystem.Compute(global.JDay,global.ThePlace);
	}
	if (global.FlagVeryFastTime)							// very fast time animation
	{	global.JDay+=MINUTE*60*24;
		_SolarSystem.Compute(global.JDay,global.ThePlace);
	}
}


// ****************  Change the coord system to altazimutal  *********
void Switch_to_altazimutal(void)
{	glLoadIdentity();
	RADE_to_XYZ(global.AzVision, global.AltVision, global.XYZVisionAltAz);
	// Calc the equatorial coordinate of the direction of vision wich was in Altazimuthal coordinate
	global.XYZVision=global.XYZVisionAltAz;
	AltAz_to_equ(global.XYZVision,global.RaZenith,global.DeZenith);
	// Change it... the vertical direction is always the vector 0,1,0 in Altazimuthal coordinate
	gluLookAt(0, 0, 0, 			// Observer position
		      global.XYZVisionAltAz[0],global.XYZVisionAltAz[1],global.XYZVisionAltAz[2],	// direction of vision
			  0,1,0);			// Vertical vector
}

// *****************  Change the coord system to equatorial  **********
void Switch_to_equatorial(void)
{	glLoadIdentity();
	Switch_to_altazimutal();
	// Make the 2 rotations to transform the Altazimuthal system to Equatorial system
	glRotatef(90-global.DeZenith*180/PI,1,0,0);
	glRotatef(global.RaZenith*180/PI,0,1,0);
}
